package shaderPrograms;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.nio.FloatBuffer;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL20;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;

public abstract class SuperShader {
	
	private int vertexShaderID;
	private int fragmentShaderID;
	private int shaderProgramID;
	
	private static FloatBuffer matrixBuffer = BufferUtils.createFloatBuffer(16);
	
	public SuperShader(String vertexShaderFile, String fragmentShaderFile) {
		vertexShaderID = readShaderFile(vertexShaderFile, GL20.GL_VERTEX_SHADER);
		fragmentShaderID = readShaderFile(fragmentShaderFile, GL20.GL_FRAGMENT_SHADER);
		shaderProgramID = GL20.glCreateProgram();
		
		GL20.glAttachShader(shaderProgramID, vertexShaderID);
		GL20.glAttachShader(shaderProgramID, fragmentShaderID);
		
		GL20.glLinkProgram(shaderProgramID);
		GL20.glValidateProgram(shaderProgramID);
		getAllUniformLocations();
		
	}
	
	protected abstract void getAllUniformLocations();
	
	protected int getUniformLocation(String uniformName) {
		return GL20.glGetUniformLocation(shaderProgramID, uniformName);
	}
	
	private static int readShaderFile(String filename, int shaderType) {
		
		StringBuilder shaderSourceCode = new StringBuilder();
		
		try {
			
			BufferedReader br = new BufferedReader(new FileReader(filename));
			String line = br.readLine();
		
			while (line != null) {
				shaderSourceCode.append(line);
				shaderSourceCode.append(System.lineSeparator());
				line = br.readLine();
			}
			br.close();
		} catch(IOException e) {
			System.err.println("Could not read file!");
			e.printStackTrace();
			System.exit(-1);
		}
		
		int shaderID = GL20.glCreateShader(shaderType);
		
		GL20.glShaderSource(shaderID, shaderSourceCode);
		GL20.glCompileShader(shaderID);
		
		if (GL20.glGetShaderi(shaderID, GL20.GL_COMPILE_STATUS) == GL11.GL_FALSE) {
			System.out.println(GL20.glGetShaderInfoLog(shaderID, 500));
			System.err.println("Could not compile shader.");
			System.exit(-1);
		}
		return shaderID;
	}
	
	protected void loadMatrix(int location, Matrix4f matrix) {
		matrix.store(matrixBuffer);
		matrixBuffer.flip();
		GL20.glUniformMatrix4(location, false, matrixBuffer);
	}
	
	public void start() {
		GL20.glUseProgram(shaderProgramID);
	}

	public void stop() {
		GL20.glUseProgram(0);
	}
	
	public void cleanUp() {
		stop();
		GL20.glDetachShader(shaderProgramID, vertexShaderID);
		GL20.glDetachShader(shaderProgramID, fragmentShaderID);
		GL20.glDeleteShader(vertexShaderID);
		GL20.glDeleteShader(fragmentShaderID);
		GL20.glDeleteProgram(shaderProgramID);
		
	}

	protected void loadVector(int location, Vector3f vector) {
		GL20.glUniform3f(location, vector.x, vector.y, vector.z);
	}
	
}
